@@ -10,7 +10,7 @@ class NODETYPES:
 
 class Node:
     """
-    A recursive tree structure, supporting both dict and list nodes. New children nodes can be read and written as attributes, and dynamically becomes nodes themselves. 
+    A recursive tree structure, supporting both dict and list nodes. New children nodes can be read and written as attributes, and dynamically become nodes themselves. 
 
     Example
     -------------
@@ -153,8 +153,6 @@ class Node:
         """
         if name in ("_value", "_frozen", "_sealed"):
             return super().__setattr__(name, value)
-        if name in {"serialize", "get", "append"}:
-            raise AttributeError(f"Attribute '{name}' is read-only")
         if self.__nodetype == NODETYPES.UNDEFINED:
             if self._frozen:
                 raise AttributeError(
@@ -180,15 +178,7 @@ class Node:
         """
         Remove an attribute by name
         """
-        if name in {
-            "__nodetype",
-            "_value",
-            "_frozen",
-            "_sealed",
-            "serialize",
-            "get",
-            "append",
-        }:
+        if name in {"__nodetype", "_value", "_frozen", "_sealed"}:
             raise AttributeError(f"Attribute '{name}' is read-only")
         if self.__nodetype == NODETYPES.UNDEFINED:
             raise AttributeError(f"undefined node has no attribute '{name}'")
@@ -392,6 +382,152 @@ class Node:
             return self._value[key]
         return default
 
+    def deepget(self, keys, default=None):
+        """
+        Returns the value at the end of the keys' path, or 
+        default if such path raises an KeyError or IndexError
+
+        Parameters
+        ----------
+        keys : iterable
+            list of keys
+        default : *
+            default value if keys' path does not exists
+
+        Example
+        -------
+        >>> tree = easytree.Tree({"address":{"city":"New York"}})
+        >>> tree.deepget(("address","city"))
+        "New York"
+        >>> tree.deepget(("address","country"), "US")
+        "US"
+        """
+        node = self
+        for key in keys:
+            if isinstance(node, Node):
+                if node._Node__nodetype == NODETYPES.UNDEFINED:
+                    return default
+                if node._Node__nodetype == NODETYPES.DICT:
+                    if key not in node:
+                        return default
+            try:
+                node = node[key]
+            except (KeyError, IndexError):
+                return default
+        return node
+
+    def pop(self, *args, **kwargs):
+        """
+        Removes (in-place) the item at given key/index and returns the corresponding value.
+
+        Note
+        ----
+        Calling pop on an undefined node 
+
+        Example
+        -------
+        >>> tree = easytree.Tree({"name":"Bob","numbers":[1,3,5], "address":{"country":"US"}})
+        >>> tree.pop("name")
+        "Bob"
+        >>> tree
+        easytree.Tree({"numbers":[1,3,5], "address":{"country":"US"}})
+        >>> tree.numbers.pop()
+        5
+        >>> tree
+        easytree.Tree({"numbers":[1,3], "address":{"country":"US"}})
+        """
+        if self.__nodetype == NODETYPES.UNDEFINED:
+            if len(args) + len(kwargs) == 2:
+                return {}.pop(*args, **kwargs)
+            if len(args) + len(kwargs) == 0:
+                return [].pop()
+            if len(args) == 0 and len(kwargs) == 1 and "index" in kwargs:
+                return [].pop(**kwargs)
+            return {}.pop(*args, **kwargs)
+        return self._value.pop(*args, **kwargs)
+
+    def keys(self):
+        """
+        Returns the keys at the dict node
+
+        Returns
+        -------
+        dict_keys
+
+        Raises
+        ------
+        AttributeError
+            if node is a list node
+        """
+        if self.__nodetype == NODETYPES.LIST:
+            raise AttributeError("list node has no attribute 'keys'")
+        if self.__nodetype == NODETYPES.UNDEFINED:
+            return {}.keys()
+        return self._value.keys()
+
+    def values(self):
+        """
+        Returns the values at the dict node
+
+        Returns
+        -------
+        dict_values
+
+        Raises
+        ------
+        AttributeError
+            if node is a list node
+        """
+        if self.__nodetype == NODETYPES.LIST:
+            raise AttributeError("list node has no attribute 'values'")
+        if self.__nodetype == NODETYPES.UNDEFINED:
+            return {}.values()
+        return self._value.values()
+
+    def items(self):
+        """
+        Returns the items at the dict node
+
+        Returns
+        -------
+        dict_items
+
+        Raises
+        ------
+        AttributeError
+            if node is a list node
+        """
+        if self.__nodetype == NODETYPES.LIST:
+            raise AttributeError("list node has no attribute 'items'")
+        if self.__nodetype == NODETYPES.UNDEFINED:
+            return {}.items()
+        return self._value.items()
+
+    def update(self, other):
+        """
+        Updates (in place) the dict node with other mapping
+
+        Note 
+        ----
+        An undefined node will be cast as a dict node
+
+        Returns
+        -------
+        None
+
+        Raises
+        ------
+        AttributeError
+            if node is a list node
+        """
+        if self.__nodetype == NODETYPES.LIST:
+            raise AttributeError("list node has no attribute 'items'")
+        if self.__nodetype == NODETYPES.UNDEFINED:
+            self._value = Node(other)._value
+        else:
+            self._value.update(Node(other)._value)
+        return
+
     def serialize(self):
         """
         Recursively converts itself to a native python type (dict, list or None).
